#!/usr/bin/env python 
# ----------------------
# Name: ttvdb.py
# Python Script
# Author: 	R.D. Vaughan
# Purpose: 	This python script is intended to perform TV series data lookups based on information found on
# 			the http://thetvdb.com/ website. It follows the MythTV standards set for the movie data
#           lookups. e.g. the perl script "tmdb.pl" used to access themoviedb.com
#			This script uses the python module tvdb_api.py (v0.5 or higher) found at 
#			http://pypi.python.org/pypi?%3Aaction=search&term=tvnamer&submit=search thanks 
#			to the authors of this excellant module.
#			The tvdb_api.py module uses the full access XML api published by thetvdb.com see:
#			http://thetvdb.com/wiki/index.php?title=Programmers_API
#			Users of this script are encouraged to populate thetvdb.com with TV show information, posters,
#			fan art and banners. The richer the source the more valuable the script.
#			This python script was modified based on the "tvnamer.py" created by "dbr/Ben" who is also
#           the author of the "tvdb_api.py" module. "tvnamer.py" is used to rename avi files with 
#           series/episode information found at thetvdb.com
# Command example:
# See help (-u and -h) options
#
# Design:
#	1) Verify the command line options (display help or version and exit)
#	2) Verify that thetvdb.com has the series or series_season_ep being requested exit if does not exit
#	3) Find the requested information and send to stdout if any found
#
#
# License:Creative Commons GNU GPL v2
# (http://creativecommons.org/licenses/GPL/2.0/)
#-------------------------------------
__title__ ="thetvdb.com Query";
__author__="R.D.Vaughan"
__version__="v0.8.0"	 	# Version .1 Initial development 
							# Version .2 Add an option to get season and episode numbers from ep name 
							# Version .3 Cleaned up the documentation and added a usage display option
							# Version .4 Added override formating of the number option (-N) 
							# Version .5 Added a check that tvdb_api.py dependancies are installed  
							# Version .6 Added -M Series List functionality  
							# Version .7 Added -o Series name override file functionality  
							# Version .8 Removed a dependancy, fixed caching for multiple users and
							#			 used better method of supporting the -M option with tvdb_api

usage_txt='''
This script fetches TV series information from theTVdb.com web site. The script conforms to MythTV's 
query engines for Movie Data standards.
NOTE: In the case of multiple queries the display order will always be:
    Data            - every initial line of an episode's data starts with 'Season:'
    Numbers         - Used as a single option not valid as a multi-option parameter 
    List            - Used as a single option not valid as a multi-option parameter 
    Poster URL(s)   - initial line is 'poster' then the URL(s) follow on the next line(s)
    Fan art URL(s)  - initial line is 'fanart' then the URL(s) follow on the next line(s)
    Banner URL(s)   - initial line is 'banner' then the URL(s) follow on the next line(s)
NOTE: If a query request has no results then no data is output for that query.
NOTE: All errors starts with a '! ' and a return code of "False". 
      Queries with no results have no output but have a return code of "True".
	
Options include:
  -h, --help        : Display help
  -u, --usage       : Examples on using this script
  -v, --version     : Prints the script title, version number and script author
  -d, --debug       : Verbose, debug information
  -r, --raw         : Display raw XML information returned from various queries
  -i, --interactive : Allows command line interaction when selecting from a number of possible series
                      Default selects the first series matching the series name so be specific.
  -o, --override    : Use information in a file if found at ~/.tvdb/override.txt to override series names
                      See the example file "override_example.txt" for details
  -M, --list        : Returns a list of matching series
  -D, --data        : Returns the series episode information
  -B, --backdrop    : Returns the URL(s) to the series banner graphics
  -F, --fanart      : Returns the URL(s) to the series fan art graphics
  -P, --poster      : Returns the URL(s) to the series poster graphics
  -N, --numbers     : Returns the season number and episode number from an episode name 
 NOTE: for options (MPFBDN) an empty string is returned to stdout if no information was found

2nd arg   : (not required for -h, -v or -u) 
    The name of the series or series/season or series/season/episode query request
    -B (banner graphics requests) are series specific 
    -D (data requests) can be specific to all seasons or all episodes for 
       a season or a specific season and episode
    -N (season and episode numbers) must have an series and episode names as input parameters
    NOTE: There are no identifiers for URLs output if ONLY one type has been selected (e.g -B versus -BF)
    NOTE: -PFB (poster, fan art and banner graphics requests) are only by Series 
 
Command examples:
(Return the banner graphics for a series)
> ttvdb -B "Sanctuary"
http://www.thetvdb.com/banners/seasons/80159-1.jpg
http://www.thetvdb.com/banners/seasons/80159-1-2.jpg
http://www.thetvdb.com/banners/seasons/80159-1-4.jpg
http://www.thetvdb.com/banners/seasons/80159-1-3.jpg

(Return the posters, banners and fan art for a series)
> ttvdb -PFB "Sanctuary"
poster:
http://www.thetvdb.com/banners/posters/80159-1.jpg
fanart:
http://www.thetvdb.com/banners/fanart/original/80159-5.jpg
http://www.thetvdb.com/banners/fanart/original/80159-1.jpg
banner:
http://www.thetvdb.com/banners/seasons/80159-1.jpg

(Return a season and episode numbers) Note: Any other options are ignored e.g. (-BFP ... etc) 
> ttvdb -N "Eleventh Hour (US)" "H2O"
[01x10]

(Return a season and episode numbers with an override of the default output format)
(WARNING: There is no editing of an override output format, two '%d' must be included)
> ttvdb -N "Eleventh Hour (US)" "H2O" "S%de%02d"
S1e10

(Return a season and episode numbers using the override file to accurately id the series as the US version)
> ttvdb -oN "Eleventh Hour" "H2O"
[01x10]

(Return the meta data for a specific series/season/episode)
> ttvdb -D "Sanctuary" 0 1
Season: 0
Episode: 1
Title:Webisode 1
ReleaseDate:2007-05-14
Director:Martin Wood
Plot:While tracking a young boy with dangerous powers, Dr. Helen Magnus encounters Will Zimmerman, 
a psychiatric resident who may have what it takes to become her new protege.
UserRating:8.0
Writers:Damian Kindler|Martin Wood
Episode Image:http://images.thetvdb.com/banners/episodes/80159-328953.jpg
Language:en

(Return a list of "thetv.com series id and series name" that contain specific search word(s) )
(!! Be careful with this option as poorly defined search words can result in large lists being returned !!)
> ttvdb.py -M "night a"
74382:Love on a Saturday Night
71476:Star For A Night

'''
# Episode keys that can be used in a episode data/information search. 
# Not all keys are currently being used. 
'''
seasonid
episodenumber
seasonnumber
language
rating
overview
writer
filename
lastupdated
episodename
director
combined_episodenumber
firstaired
combined_season
gueststars
epimgflag
seriesid
id
'''


# System modules
import sys, os, subprocess, re
from optparse import OptionParser

# Verify that tvdb_api.py, tvdb_ui.py and tvdb_exceptions.py are available
try:
	import tvdb_ui
	# thetvdb.com specific modules
	from tvdb_api import (tvdb_error, tvdb_shownotfound, tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_episodenotfound, tvdb_attributenotfound, tvdb_userabort)
	from tvdb_api import Tvdb
except:
	print '''
The modules tvdb_api.py (v0.50 or greater), tvdb_ui.py and tvdb_exceptions.py must be installed. 
They should have been included with the distribution of ttvdb.py.
'''
	sys.exit(False)

# Global variables
# Keys and titles used for episode data (option '-D')
data_keys	=['seasonnumber','episodenumber','episodename','firstaired','director','overview','rating','writer','filename','language' ]
data_titles	=['Season:','Episode:','Title:','ReleaseDate:','Director:','Plot:','UserRating:','Writers:','Episode Image:','Language:' ]
poster_key='poster'
fanart_key='fanart'
banner_key_series='series'
banner_key_season='season'
# Unfortunitely the episodes image URL string is returned without the base URL information
# Hopefully in the future the tv_api.py will return a fully qualified URL
banner_base_URL='http://images.thetvdb.com/banners/'

# Cache directory name specific to the user. This avoids permission denied error with a common cache dirs 
cache_dir="/tmp/tvdb_api_%s/" % os.geteuid() 

# Verify that a Series or Series and Season exists on thetvdb.com
def searchseries(t, opts, series_season_ep):
	series_name=''
	if opts.override == True and override.has_key(series_season_ep[0]):
		series_name=override[series_season_ep[0]][0] 	# Override series name
	else:
		series_name=series_season_ep[0] 				# Leave the series name alone
	try:
		# Search for the series or series & season or series & season & episode
		if len(series_season_ep)>1:
			if len(series_season_ep)>2: 							# series & season & episode
				seriesfound=t[ series_name ][ int(series_season_ep[1]) ][ int(series_season_ep[2]) ]
			else:
				seriesfound=t[ series_name ][ int(series_season_ep[1]) ] # series & season
		else:
			seriesfound=t[ series_name ]	# Series only
	except tvdb_shownotfound:
		# No such show found.
		# Use the show-name from the files name, and None as the ep name
		sys.stderr.write("! Warning: Series (%s) not found\n" % (
			series_name )
		)
		sys.exit(False)
	except (tvdb_seasonnotfound, tvdb_episodenotfound, tvdb_attributenotfound):
		# The season, episode or name wasn't found, but the show was.
		# Use the corrected show-name, but no episode name.
		if len(series_season_ep)>2: 
			sys.stderr.write("! Warning: For Series (%s), season (%s) or Episode (%s) not found \n" % (
				series_name, series_season_ep[1], series_season_ep[2] )
			)
		else:
			sys.stderr.write("! Warning: For Series (%s), season (%s) not found \n" % (
				series_name, series_season_ep[1] )
			)
		sys.exit(False)
	except tvdb_error, errormsg:
		# Error communicating with thetvdb.com
		sys.stderr.write(
			"! Warning: Error contacting www.thetvdb.com:\n%s\n" % (errormsg)
		)
		sys.exit(False)
	except tvdb_userabort, errormsg:
		# User aborted selection (q or ^c)
		print "\n", errormsg
		sys.exit(False)
	else:
		if opts.raw==True:
			print "="*20
			print "Raw Series Data:\n"
			if len(series_season_ep)>1:
				print t[ series_name ][ int(series_season_ep[1]) ]
			else:
				print t[ series_name ]
			print "="*20
		return(seriesfound)
# end searchseries

# Retrieve Poster or Fan Art or Banner graphics URL(s)
def get_graphics(t, opts, series_season_ep, banner_type, single_option):
	banners='_banners'
	series_name=''
	if opts.override == True and override.has_key(series_season_ep[0]):
		series_name=override[series_season_ep[0]][0] 	# Override series name
	else:
		series_name=series_season_ep[0] 				# Leave the series name alone
	graphics = t[series_name][banners][banner_type].keys()
	graphicsURLs=[]
	if single_option==False:
		graphicsURLs.append(banner_type+':')
	for g in graphics:
		ids=t[series_name][banners][banner_type][g].keys()
		for i in ids:
			graphicsURLs.append(banner_base_URL+t[series_name][banners][banner_type][g][i]['BannerPath'])
	if opts.debug == True:
		print "\nGraphics:\n", graphicsURLs
	return(graphicsURLs)
# end get_graphics

# Get Series Episode data by season
def Getseries_episode_data(t, opts, series_season_ep):
	args = len(series_season_ep)
	series_name=''
	if opts.override == True and override.has_key(series_season_ep[0]):
		series_name=override[series_season_ep[0]][0] 	# Override series name
	else:
		series_name=series_season_ep[0] 				# Leave the series name alone
	seasons=t[series_name].keys()	# Get the seasons for this series
	for season in seasons:
		if args > 1:						# If a season was specified skip other seasons
			if season != int(series_season_ep[1]):
				continue
		seasons_key_values=[]
		episodes=t[series_name][season].keys()	# Get the episodes for this season
		for episode in episodes:			# If an episode was specified skip other episodes
			if args > 2:
				if episode != int(series_season_ep[2]):
					continue
			available_keys=t[series_name][season][episode].keys()
			key_values=[]
			for values in data_keys:	# Initialize an array for each possible data element for 
				key_values.append('')	# each episode within a season
			for key in available_keys:
				try:
					i = data_keys.index(key)	# Include only specific episode data 
				except ValueError:
					continue
				if key == 'filename':			# Create a full URL for the Episode image graphic
					key_values[i]=banner_base_URL+t[series_name][season][episode][key]
				else:
					key_values[i]=t[series_name][season][episode][key]
			seasons_key_values.append(key_values)
		for values in seasons_key_values: 		# Output episode data
			index = 0
			for key in data_titles:
				print '%s%s'% (data_titles[index], values[index])
				index+=1
# end Getseries_episode_data

# Get Series Season and Episode numbers
def Getseries_episode_numbers(t, opts, series_season_ep):
	series_name=''
	ep_name=''
	if opts.override == True and override.has_key(series_season_ep[0]):
		series_name=override[series_season_ep[0]][0] 		# Override series name
		ep_name=series_season_ep[1]
		if len(override[series_season_ep[0]][1]) != 0:		# Are the search-replace strings?
			for edit in override[series_season_ep[0]][1]:
				ep_name=ep_name.replace(edit[0],edit[1])	# Edit episode name for each set of strings 
	else:
		series_name=series_season_ep[0] 				# Leave the series name alone
		ep_name=series_season_ep[1]						# Leave the episode name alone

	season_ep_num=t[series_name].search(ep_name)
	if len(season_ep_num) != 0:
		print season_and_episode_num % (int(season_ep_num[0]['seasonnumber']), int(season_ep_num[0]['episodenumber']))
# end Getseries_episode_numbers

# Set up a custom interface to get all searies matching a partial series name
class returnAllSeriesUI(tvdb_ui.BaseUI):
	def __init__(self, config, log):
		self.config = config
		self.log = log

	def selectSeries(self, allSeries):
		return allSeries
# ends returnAllSeriesUI

# Fill out series name override dictionary from "~/.tvdb/override.txt" file 
def initialize_override_dictionary(override_filename):
	series_name=0
	sid=1
	myfile = open(override_filename)	# Open for output (to read only)
	aList = myfile.readlines( ) 		# Read entire file into list of line strings
	myfile.close( ) 					# Flush output buffers to disk
										# Get access to tv_api functions
	tvdb = Tvdb(banners=False, debug = False, interactive = False, cache = cache_dir, custom_ui=returnAllSeriesUI)
	for rec in aList:
		rec = rec.rstrip()				# Remove \n characters from the end of each record
		if len(rec) == 0:				# Skip empty records
			continue
		if rec[0] == '#':				# Skip comment lines
			continue
		parts = rec.split('\t')			# Split off each value deliniated by a '\t' (tab character)
		if len(parts) == 0:				# Skip an errent empty record that contained a tab character	
			continue
		if (len(parts) == 1 and parts[0]==''): # Last record can be empty, skip it
			continue
		if len(parts) % 2 != 0 :		# The number of fields must be even
			sys.stdout.write("! Invalid Override Record (not an even number of fields) Total fields:%d\n" % len(parts))							# Version .7 Added -o Series name override file functionality  

			print parts
			sys.exit(False)

		for i in range(len(parts)):		# Remove all double quotes from around each field
			parts[i]=parts[i].replace('"','')

		try:					# Check that the SID (Series id) is numeric
		    dummy = int(parts[sid])
		except:
			sys.stdout.write("! Invalid SID (not numeric) [%s]\n" % parts[sid])
			print parts
			sys.exit(False)
		# Make sure that the series name is not empty or all blanks
		if len(parts[series_name].replace(' ','')) == 0:
			sys.stdout.write("! Invalid Series name (must have some non-blank characters) [%s]\n" % parts[series_name])
			print parts
			sys.exit(False)

		try:
			allSeries=tvdb._getSeries(parts[series_name])
		except:
			try:
				name=parts[series_name].replace('/',' ') 	# Remove non-alphabetic characters
				if name.index(' ') != 0:
					name=name[:name.index(' ')]				# Extract the first work in the series name
				allSeries=tvdb._getSeries(name)
			except:
				sys.stdout.write("! Invalid Series (no matches found in thetvdb,com) [%s]\n" % parts[series_name])
				print parts
				sys.exit(False)
	
		tmp_override_series_name = ''
		for series_name_sid in allSeries: 					# finding matching sid number
			if series_name_sid['sid'] == parts[sid]:
				tmp_override_series_name = series_name_sid['name']
				break
		if tmp_override_series_name == '':	
			tmp_override_series_name = parts[series_name]	# No override found for SID

		ep_search_replace=[]
		if len(parts) != 2:			# Check if there are any episode name search-replace directives 
			ep_edit_pairings=parts[2:]
			for i in (range(len(ep_edit_pairings)/2)):
				if i == 0:
					j=i
				else:
					j=i*2
				if ep_edit_pairings[j] == '':		# Empty search strings are invalid 
					sys.stdout.write("! Invalid Search string (must not be empty) [%s] for replace [%s]\n" % (ep_edit_pairings[j], ep_edit_pairings[j+1]))
					print parts
					sys.exit(False)
				ep_search_replace.append([ep_edit_pairings[j],ep_edit_pairings[j+1]])

		# Add override details for this series
		override[parts[series_name]]=[tmp_override_series_name,ep_search_replace]
	return
# end initialize_override_dictionary

def main():
	parser = OptionParser(usage="%prog usage: ttvdb -hdrvioMPFBD [parameters]\n <series name or 'series and season number' or 'series and season number and episode number'>")

	parser.add_option(  "-d", "--debug", action="store_true", default=False, dest="debug",
                        help="Show debugging info")
	parser.add_option(  "-r", "--raw", action="store_true",default=False, dest="raw", 
                        help="Dump raw data only")
	parser.add_option(  "-u", "--usage", action="store_true", default=False, dest="usage",
                        help="Display examples for executing the ttvdb script")
	parser.add_option(  "-v", "--version", action="store_true", default=False, dest="version",
                        help="Display version and author")
	parser.add_option(  "-i", "--interactive", action="store_true", default=False, dest="interactive",
                        help="Interaction mode (allows selection of a specific Series)")
	parser.add_option(  "-o", "--override", action="store_true", default=False, dest="override",
                        help="Use a file if found at ~/.tvdb/override.txt to override series names")
	parser.add_option(  "-M", "--list", action="store_true", default=False, dest="list",
                        help="(Not ACTIVE) Get Series list")
	parser.add_option(  "-P", "--poster", action="store_true", default=False, dest="poster",
                        help="Get Series Poster URL")
	parser.add_option(  "-F", "--fanart", action="store_true", default=False, dest="fanart",
                        help="Get Series fan art URL(s)")
	parser.add_option(  "-B", "--backdrop", action="store_true", default=False, dest="banner",
                        help="Get Series banner/backdrop URL(s)")
	parser.add_option(  "-D", "--data", action="store_true", default=False, dest="data",
                        help="Get Series data")
	parser.add_option(  "-N", "--numbers", action="store_true", default=False, dest="numbers",
                        help="Get Season and Episode numbers")

	opts, series_season_ep = parser.parse_args()

	if opts.debug == True:
		print "opts", opts
		print "\nargs", series_season_ep

	# Process version command line requests
	if opts.version == True:
		sys.stdout.write("\nTitle: (%s); Version: (%s); Author: (%s)\n" % (
		__title__, __version__, __author__ ))
		sys.stdout.write("This python script uses the included python modules tvdb_api.py (v0.6 or higher) found at\n\
'http://pypi.python.org/pypi?%3Aaction=search&term=tvnamer&submit=search' thanks to the authors 'dbr/Ben' for this excellant module.\n\n\
The tvdb_api.py module uses the full access XML api published by thetvdb.com see:\n\
'http://thetvdb.com/wiki/index.php?title=Programmers_API'\n\n\
Users of this script are encouraged to populate thetvdb.com with TV show information, posters,\n\
fan art and banners. The richer the source the more valuable the script.\n\n"
		)
		sys.exit(True)

	# Process usage command line requests
	if opts.usage == True:
		sys.stdout.write(usage_txt)
		sys.exit(True)

	if len(series_season_ep) == 0:
		parser.error("! No series or series_season_ep supplied")
		sys.exit(False)

	# Default output format of season and episode numbers
	global season_and_episode_num
	season_and_episode_num='[%02dx%02d]'

	if opts.numbers == False:
		if len(series_season_ep) > 1:
			if series_season_ep[1].lstrip('0123456789') != "":
				parser.error("! Season is not numeric")
				sys.exit(False)
		if len(series_season_ep) > 2:
			if series_season_ep[2].lstrip('0123456789') != "":
				parser.error("! Episode is not numeric")
				sys.exit(False)
	else:
		if len(series_season_ep) < 2:
			parser.error("! An Episode name must be included")
			sys.exit(False)
		if len(series_season_ep) == 3:
			season_and_episode_num = series_season_ep[2]	# Override default output format

	if opts.debug == True:
		print series_season_ep

	if opts.debug == True:
		print "#"*20
		print "# series_season_ep array(",series_season_ep,")"
	
	if opts.debug == True:
		print "#"*20
		print "# Starting tvtvb"
		print "# Processing (%s) Series" % ( series_season_ep[0] )

	# Access thetvdb.com API with banners (Posters, Fanart, banners) data retrieval enabled
	if opts.list ==True: 
		t = Tvdb(banners=False, debug = opts.debug, cache = cache_dir, custom_ui=returnAllSeriesUI)
	elif opts.interactive == True:
		t = Tvdb(banners=True, debug=opts.debug, interactive=True,  select_first=False, cache=cache_dir)
	else:
		t = Tvdb(banners=True, debug = opts.debug, cache = cache_dir)

	if opts.debug == True:
		print "# ..got tvdb mirrors"
		print "# Start to process series or series_season_ep"
		print "#"*20

	# Fetch a list of matching series names 
	if (opts.list ==True ): 
		allSeries=t._getSeries(series_season_ep[0])
		for series_name_sid in allSeries: # list search results
			print "%s:%s" % (series_name_sid['sid'], series_name_sid['name'])
		sys.exit(True) # The Series list option (-M) is the only option honoured when used

	global override
	override={}			# Initialize series name override dictionary
	# If the use wants Series name overrides and a override file exists then create an overide dictionary
	if opts.override == True:
		override_filename="%s/.tvdb/override.txt" % os.path.expanduser('~')
		if os.path.exists(override_filename) == 1: # Do overrides exist?
			initialize_override_dictionary(override_filename)
	if len(override) == 0:
		opts.override = False	# Turn off the override option as there is nothing to override
	
	# Verify that thetvdb.com has the desired series_season_ep exits this module if series_season_ep not found
	if opts.numbers == False:
		seriesfound=searchseries(t, opts, series_season_ep)
	else:
		x=[]
		x.append(series_season_ep[0])	# Only use series name in check
		seriesfound=searchseries(t, opts, x)

	# Dump information accessable for a Series and ONLY first season of episoded data
	if opts.debug == True:
		print "#"*20
		print "# Starting Raw keys call"
		print "Lvl #1:" 								# Seasons for series
		x = t[series_season_ep[0]].keys()
		print t[series_season_ep[0]].keys()
		print "#"*20
		print "Lvl #2:"									# Episodes for each season
		for y in x:
			print t[series_season_ep[0]][y].keys()
		print "#"*20
		print "Lvl #3:"									# Keys for each episode within the 1st season
		z = t[series_season_ep[0]][1].keys()
		for aa in z:
			print t[series_season_ep[0]][1][aa].keys()
		print "#"*20
		print "Lvl #4:"									# Available data for each episode in 1st season
		for aa in z:
			codes = t[series_season_ep[0]][1][aa].keys()
			print "\n\nStart:"
			for c in codes:
				print "="*50
				print 'Key Name=('+c+'):'
				print t[series_season_ep[0]][1][aa][c]
				print "="*50
		print "#"*20
		sys.exit (True)

	if opts.numbers == True:				# Fetch and output season and episode numbers
		Getseries_episode_numbers(t, opts, series_season_ep)
		sys.exit(True) # The Numbers option (-N) is the only option honoured when used

	if opts.data == True:				# Fetch and output episode data
		Getseries_episode_data(t, opts, series_season_ep)

	# Fetch the requested graphics URL(s)
	if opts.debug == True:
		print "#"*20
		print "# Checking if Posters, Fanart or Banners are available"
		print "#"*20
	if opts.override == True and override.has_key(series_season_ep[0]):
		banners_keys = t[override[series_season_ep[0]][0]]['_banners'].keys()
	else:
		banners_keys = t[series_season_ep[0]]['_banners'].keys()
	banner=poster=fanart= False

	for x in banners_keys:				# Determine what type of graphics is available
		if x == fanart_key:
			fanart=True
		elif x== poster_key:
			poster=True
		elif x== banner_key_series:
			banner=True
		elif x== banner_key_season:
			banner=True

	# Make sure that some graphics URL(s) (Posters, FanArt or Banners) are available
	if ( fanart!=True and poster!=True and banner!=True ):
		sys.exit(True)
	
	if opts.debug == True:
		print "#"*20
		print "# One or more of Posters, Fanart or Banners are available"
		print "#"*20

	# Determine if graphic URL identifaction output is required	
	y=0
	single_option=True
	if opts.poster==True:
		y+=1
	if opts.fanart==True:
		y+=1
	if opts.banner==True:
		y+=1
	if y > 1:							
		single_option=False

	if (poster==True and opts.poster==True and opts.raw!=True):	# Get posters and send to stdout
		for p in get_graphics(t, opts, series_season_ep, poster_key, single_option):
			print p

	if (fanart==True and opts.fanart==True and opts.raw!=True):	# Get Fan Art and send to stdout
		for f in get_graphics(t, opts, series_season_ep, fanart_key, single_option):
			print f

	if (banner==True and opts.banner==True and opts.raw!=True):	# Get Banners and send to stdout
		x=False
		if len (series_season_ep) == 1:			# Determine which type of banner should be retrieved
			series_season_ep.append('1')		# Banners always require a season number
		type_banner = banner_key_series
		for b in get_graphics(t, opts, series_season_ep, type_banner, single_option):
			if (x==False and single_option==False):
				print 'banner:'		# Change output identifier from 'series' or 'season' to 'banner'
				x=True
				continue
			print b

	if opts.debug == True:
		print "#"*20
		print "# Processing complete"
		print "#"*20
	sys.exit(True)
#end main

if __name__ == "__main__":
	main()
